#Add on  to switch between Eloquence and SAPI5  synthesizes
#for SAPI5, use Sao Mai and Phuong Nam (VOS) synthesizers
# Author: Dung Nguyen <dungnv1984@gmail.com>
# Date: November  2013
# Key combination:  
#NVDA + ` : switch between English and Vietnamese voice
# control + NVDA + `: switch between voices in Phuong Nam
# control + Alt + ` : Switch voices in Sao Mai (Mai dung and Minh Du)

import globalPluginHandler
import speech
import time
import ui
import globalVars
import config
import os.path
from logHandler import log

# define string constant
elo = "eloquence"
sapi= "sapi5"
sm1 = "sm1"
sm2 = "sm2"
speechSession = "speech"
voice = "voice"
vos1= "vos1"
vos2 ="vos2"
vos3 = "vos3"
saomai = "saomai"
vos = "vos"
active = "active"
messageFile = "messages.ini"
vnvoiceFile = "vnvoice.ini"
rate = "rate"
pitch = "pitch"
volume = "volume"
empty = ""
encoding = "utf8"

class GlobalPlugin (globalPluginHandler.GlobalPlugin):
	#ui.message("build successfully")
	# Binding a key for script
	__gestures={"kb:NVDA+`": "switchSynth",
	"kb:alt+control+`": "selectSaoMai",
	"kb:Control+NVDA+`": "selectVos",
	}

	#define a script to switch between Eloquence and SAPI synthesizers 
	def script_switchSynth(self, gesture):
		# save curent configuration into nvda.ini file
		config.conf.save()
		
		#get curent synthesize
		currentSynth = speech.getSynth().name
		
		#switch between synthesizers
		message = ""
		if currentSynth == elo:
			currentSynth = sapi
			# get new voice information 
			fileName = self.getVoicePath() + vnvoiceFile
			activeVoice = self.iniReadString(active, fileName)
			message = self.getVoiceMessage(activeVoice)
			
			# get current SAPI voice in NVDA.ini file
			config.conf.reset()
			currentVoice =config.conf[speechSession][sapi]["voice"]			
			
			# check if current voice is empty
			if currentVoice == empty:
				self.switchVNVoice(activeVoice)
				return
		elif currentSynth == sapi:
			currentSynth = elo
			message = "English"
		
		# switch to new synth
		#startTime  = time.clock()
		speech.setSynth(currentSynth)
		#duration  = time.clock() - startTime
		#log.info("duration time: " + str(duration))
		ui.message(message)

	#define iniReadString function
	def iniReadString(self, key, fileName):
		if  key == empty or fileName == empty :
			return empty
		if not os.path.isfile(fileName) :
			log.info( "file: " + fileName + " does not exist")
			return empty
		file = open(fileName, "r")
		content = file.read().decode(encoding)
		file.close()
		s = key + "="
		start = content.find(s)
		if start == -1 :
			return empty
		content = content [start:]
		end = content.find("\n")
		if end > 0 :
			content = content[:end]
		start = content.find("=")
		content = content[start+1 :]
		content = content.strip()
		return content

			
	# function to read an integer key
	def iniReadInt(self, key, filePath):
		value = self.iniReadString(key, filePath)
		return int(value)
		
	#function to return path of voice directory
	def getVoicePath(self):
		return globalVars.appArgs.configPath + "\\addons\\switchVNSynth\\globalPlugins\\vnVoices\\"
	#function get voice setting and update to nvda.ini file
	def getVoiceSettings(self, voiceName):
		fileName = self.getVoicePath() + voiceName + ".ini"
		config.conf.reset()
		
		#get current voice settings from file and set to nvda.ini file
		config.conf[speechSession][sapi][voice] = self.iniReadString(voice, fileName)
		config.conf[speechSession][sapi][rate] = self.iniReadInt(rate, fileName)
		config.conf[speechSession][sapi][pitch] = self.iniReadInt(pitch, fileName)
		config.conf[speechSession][sapi][volume] = self.iniReadInt(volume, fileName)
		config.conf.save()
	
	# function save voice setting  (get from nvda.ini file and write to a separate file)
	def saveVoiceSettings(self, currentVoice):
		config.conf.reset()
		content = "[Voice Settings]\n"
		voiceName = 		config.conf[speechSession][sapi][voice]
		rateValue = 		config.conf[speechSession][sapi][rate]
		pitchValue = 		config.conf[speechSession][sapi][pitch]
		volumeValue = 		config.conf[speechSession][sapi][volume]
		
		# add to content
		content = content + voice + "=" + voiceName + "\n"
		content = content + rate + "=" + str(rateValue) + "\n"
		content = content + pitch + "=" + str(pitchValue) + "\n"
		content = content + volume + "=" + str(volumeValue) + "\n"
		
		#write into file
		fileName = self.getVoicePath() + currentVoice + ".ini"
		file = open(fileName, "w+")
		file.write(content.encode(encoding))
		file.close()
		
	
	def switchVNVoice(self, voiceName):
		
		#get voice parameter from file and update to nvda.ini file
		self.getVoiceSettings(voiceName)
		message = self.getVoiceMessage(voiceName)
		# set new voice
		speech.setSynth(sapi)
		ui.message(message)
	
	#function to save current active voice into vnvoice.ini file
	def updateVNVoice(self, saomaiVoice, vosVoice, activeVoice):
		fileName = self.getVoicePath() + vnvoiceFile
		content = "[Vietnamese voices]" + "\r\n"
		content = content +saomai + "=" + saomaiVoice + "\r\n"
		content = content + vos + "=" + vosVoice + "\r\n"
		content = content + active + "=" + activeVoice
		
		# write into vnvoice.ini file
		file = open(fileName, "w+")
		file.write(content.encode(encoding))
		file.close()
	
	# function get voice message
	def getVoiceMessage(self, voice):
		fileName = self.getVoicePath() + messageFile
		message = self.iniReadString(voice, fileName)
		#log.info("message:" + message)
		return message
	
	# script select a voice in Sao Mai synthesizer
	def script_selectSaoMai(self, gesture):
		# get current syn
		currentSynth = speech.getSynth().name
	
	#get current Vietnamese voice info
		fileName = self.getVoicePath() + vnvoiceFile;
		vosVoice = self.iniReadString(vos, fileName)
		saomaiVoice = self.iniReadString(saomai, fileName)
		activeVoice = self.iniReadString(active, fileName)
		
		#check if current syn  is not sapi syn
		if currentSynth != sapi:
			self.switchVNVoice(saomaiVoice)
			self.updateVNVoice(saomaiVoice, vosVoice, saomaiVoice)
			return
		
		#if current syn is sapi
		if activeVoice == sm1 or activeVoice == sm2:
			if saomaiVoice == sm1:
				saomaiVoice = sm2
			else:
				saomaiVoice = sm1
		
		# save current voice setting
		self.saveVoiceSettings(activeVoice)
		# switch to new voice
		self.switchVNVoice(saomaiVoice)
		self.updateVNVoice(saomaiVoice, vosVoice, saomaiVoice)
	
	# script select a voice in VOS synthesizer
	def script_selectVos(self, gesture):
		# get current syn
		currentSynth = speech.getSynth().name
		
		# get current Vietnamese info
		fileName = self.getVoicePath() + vnvoiceFile;
		saomaiVoice = self.iniReadString(saomai, fileName)
		vosVoice = self.iniReadString(vos, fileName)
		activeVoice = self.iniReadString(active, fileName)
		
		#check if current synth is not sapi
		if currentSynth != sapi:
			self.switchVNVoice(vosVoice)
			self.updateVNVoice(saomaiVoice, vosVoice, vosVoice)
			return
			
		#check if current synth is sapi and need to switch voice in Phuong Nam
		if activeVoice != sm1 and activeVoice != sm2:
			if vosVoice == vos1:
				                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                # apply for Tram anh voice in deliver in February 2014
				#message = self.getVoiceMessage(vos1)
				#ui.message(message)
				#return
				vosVoice = vos2
			elif vosVoice == vos2:
				vosVoice = vos3
			elif vosVoice == vos3:
				vosVoice = vos1
		
		# save current voice settings
		self.saveVoiceSettings(activeVoice)
		
		# switch to new voice
		self.switchVNVoice(vosVoice)
		self.updateVNVoice(saomaiVoice, vosVoice, vosVoice)